set long 4096
set echo on
set pagesize 100
set linesize 100
set serveroutput on
pause
drop view sales_view;
CREATE VIEW sales_view AS
	SELECT   substr(country_name,1,20) country
		,substr(prod_name,1,15)    product
                ,sales.prod_id             prodid
		,calendar_year             year
		,SUM(amount_sold)          tot_amt
                ,SUM(quantity_sold)        tot_qty
		,COUNT(amount_sold)        tot_sales
		FROM sh.sales  sales join sh.times times
                         on sales.time_id = times.time_id
                              join sh.products products
                         on sales.prod_id = products.prod_id
                              join sh.customers customers
                         on sales.cust_id = customers.cust_id
                              join sh.countries countries
                         on customers.country_id = countries.country_id
 	GROUP BY country_name
                ,prod_name
                ,sales.prod_id
                ,calendar_year
        ORDER BY country
                ,product
                ,year;
--
drop materialized view sales_mview;
CREATE materialized VIEW sales_mview AS
	SELECT   substr(country_name,1,20) country
		,substr(prod_name,1,15)    product
                ,sales.prod_id             prodid
		,calendar_year             year
		,SUM(amount_sold)          tot_amt
                ,SUM(quantity_sold)        tot_qty
		,COUNT(amount_sold)        tot_sales
		FROM sh.sales sales join sh.times times
                         on sales.time_id = times.time_id
                              join sh.products products
                         on sales.prod_id = products.prod_id
                              join sh.customers customers
                         on sales.cust_id = customers.cust_id
                              join sh.countries countries
                         on customers.country_id = countries.country_id
 	GROUP BY country_name
                ,prod_name
                ,sales.prod_id
                ,calendar_year
        ORDER BY country
                ,product
                ,year;
pause
set autotrace on
select count(*) from sales_view;
select count(*) from sales_mview;
pause
ALTER SYSTEM FLUSH BUFFER_CACHE;
ALTER SYSTEM FLUSH SHARED_POOL;
select country,year,product,tot_sales
   from sales_view
   where year = '2001'
   order by tot_sales,country;
pause
ALTER SYSTEM FLUSH BUFFER_CACHE;
ALTER SYSTEM FLUSH SHARED_POOL;
select country,year,product,tot_sales
   from sales_mview
   where year = '2001'
   order by tot_sales,country;
--
set autotrace off
--
pause
DROP MATERIALIZED VIEW LOG ON sh.sales;
CREATE MATERIALIZED VIEW LOG ON sh.sales
  WITH ROWID
      (prod_id, cust_id, time_id,
       channel_id, promo_id,
       quantity_sold, amount_sold)
   INCLUDING NEW VALUES;
pause
begin
	dbms_stats.gather_table_stats(ownname=>USER,
            			      tabname=>'EMP',
				      cascade=>TRUE);
end;
/
begin
	dbms_stats.gather_table_stats(ownname=>USER,
            			      tabname=>'DEPT',
				      cascade=>TRUE);
end;
/
drop materialized view daily_dept_summary;
create materialized view daily_dept_summary
 build immediate
 refresh complete
 enable query rewrite
as
select dname
      ,count(empno) nbremps
      ,sum(coalesce(sal,0)
          +coalesce(comm,0)) totpay
     from emp e full join dept d
                 on e.deptno = d.deptno
     group by dname
     order by nbremps desc
            , totpay desc;
begin
	dbms_stats.gather_schema_stats(ownname=>USER,
				      cascade=>TRUE);
end;
/
--
set autotrace on
--
alter session set query_rewrite_enabled = true;
alter session set query_rewrite_integrity = trusted;
select dname,count(empno)
      from emp,dept
      where emp.deptno = dept.deptno
      group by dname;
--
set autotrace off
--
pause
--      
ALTER SESSION SET query_rewrite_integrity=TRUSTED; 
ALTER SESSION SET query_rewrite_enabled=FORCE; 
show parameters query 
--
-- $ORACLE_HOME\rdbms\admin\utlxrw
delete from rewrite_table;
execute dbms_mview.explain_rewrite('select dname,count(empno) from emp,dept where emp.deptno = dept.deptno group by dname');
select message from rewrite_table;
execute dbms_mview.explain_rewrite('select deptno,count(*) from emp group by deptno');
select message from rewrite_table;
--
pause
--
drop table dept_summary_tab;
create table dept_summary_tab
	as
	  select dept.deptno
            ,dname
            ,count(*) nbr_emps
            ,sum(nvl(sal,0)) tot_sal
	        from scott.emp emp
                ,scott.dept dept
	        where emp.deptno(+) = dept.deptno
	        group by dept.deptno,dname;
create materialized view dept_summary_tab
  on prebuilt table 
	with reduced precision
        refresh start with sysdate next sysdate + 1
	as
	  select dept.deptno
            ,dname
            ,count(*) nbr_emps
            ,sum(nvl(sal,0)) tot_sal
	        from scott.emp emp
                ,scott.dept dept
	        where emp.deptno(+) = dept.deptno
	        group by dept.deptno,dname;
--
pause
--
BEGIN
   DBMS_REFRESH.DESTROY (name => 'john.demogroup');
END;
/
BEGIN
   DBMS_REFRESH.MAKE (
      name => 'john.demogroup',
      list => '', 
      next_date => SYSDATE, 
      interval => 'SYSDATE + 1',
      implicit_destroy => FALSE, 
      rollback_seg => '',
      push_deferred_rpc => TRUE, 
      refresh_after_errors => FALSE);
END;
/
BEGIN
   DBMS_REFRESH.ADD (
      name => 'john.demogroup',
      list => 'john.dept_summary_mview',
      lax => TRUE);
END;
/
BEGIN
   DBMS_REFRESH.ADD (
      name => 'john.demogroup',
      list => 'john.daily_dept_summary',
      lax => TRUE);
END;
/
EXECUTE DBMS_REFRESH.REFRESH ('john.demogroup');
--
pause
--
select mview_name,rewrite_enabled,to_char(last_refresh_date,'yyyy/mm/dd hh24:mi:ss') refresh
  from user_mviews;
--
--
